@@ -16,16 +16,16 @@
  */
 const char *const appname = "PuTTY";
 
-#ifdef TELNET_DEFAULT
-const int be_default_protocol = PROT_TELNET;
-#else
 const int be_default_protocol = PROT_SSH;
-#endif
 
 const struct BackendVtable *const backends[] = {
     &ssh_backend,
     &telnet_backend,
     &rlogin_backend,
+    &supdup_backend,
     &raw_backend,
+    &sshconn_backend,
     NULL
 };
+
+const size_t n_ui_backends = 1;
