@@ -16,17 +16,17 @@
  */
 const char *const appname = "PuTTY";
 
-#ifdef TELNET_DEFAULT
-const int be_default_protocol = PROT_TELNET;
-#else
 const int be_default_protocol = PROT_SSH;
-#endif
 
 const struct BackendVtable *const backends[] = {
     &ssh_backend,
+    &serial_backend,
     &telnet_backend,
     &rlogin_backend,
+    &supdup_backend,
     &raw_backend,
-    &serial_backend,
+    &sshconn_backend,
     NULL
 };
+
+const size_t n_ui_backends = 2;
