@@ -6,6 +6,10 @@
 #include <stdio.h>
 #include "putty.h"
 
+const int be_default_protocol = -1;
+
 const struct BackendVtable *const backends[] = {
     NULL
 };
+
+const size_t n_ui_backends = 0;
