@@ -12,23 +12,11 @@ const char *const appname = "PuTTYtel";
 
 const struct BackendVtable *const backends[] = {
     &telnet_backend,
-    &rlogin_backend,
-    &raw_backend,
     &serial_backend,
+    &rlogin_backend,
+    &supdup_backend,
+    &raw_backend,
     NULL
 };
 
-/*
- * Stub implementations of functions not used in non-ssh versions.
- */
-void random_save_seed(void)
-{
-}
-
-void random_destroy_seed(void)
-{
-}
-
-void noise_ultralight(NoiseSourceId id, unsigned long data)
-{
-}
+const size_t n_ui_backends = 2;
