@@ -6,21 +6,14 @@ import re
 import struct
 from binascii import hexlify
 
+assert sys.version_info[:2] >= (3,0), "This is Python 3 code"
+
 # Expect to be run from the 'test' subdirectory, one level down from
 # the main source
 putty_srcdir = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
 
-def unicode_to_bytes(arg):
-    # Slightly fiddly way to do this which should work in Python 2 and 3
-    if isinstance(arg, type(u'a')) and not isinstance(arg, type(b'a')):
-        arg = arg.encode("UTF-8")
-    return arg
-
-def bytevals(b):
-    return struct.unpack("{:d}B".format(len(b)), b)
-def valbytes(b):
-    b = list(b)
-    return struct.pack("{:d}B".format(len(b)), *b)
+def coerce_to_bytes(arg):
+    return arg.encode("UTF-8") if isinstance(arg, str) else arg
 
 class ChildProcessFailure(Exception):
     pass
@@ -69,11 +62,14 @@ class ChildProcess(object):
         if self.debug is not None:
             self.debug.write("recv: {}\n".format(line))
         return line
+    def already_terminated(self):
+        return self.sp is None and self.exitstatus is not None
     def funcall(self, cmd, args):
         if self.sp is None:
+            assert self.exitstatus is None
             self.start()
-        self.write_line(unicode_to_bytes(cmd) + b" " + b" ".join(
-            unicode_to_bytes(arg) for arg in args))
+        self.write_line(coerce_to_bytes(cmd) + b" " + b" ".join(
+            coerce_to_bytes(arg) for arg in args))
         argcount = int(self.read_line())
         return [self.read_line() for arg in range(argcount)]
     def wait_for_exit(self):
@@ -89,18 +85,38 @@ class ChildProcess(object):
 
 childprocess = ChildProcess()
 
+method_prefixes = {
+    'val_wpoint': 'ecc_weierstrass_',
+    'val_mpoint': 'ecc_montgomery_',
+    'val_epoint': 'ecc_edwards_',
+    'val_hash': 'ssh_hash_',
+    'val_mac': 'ssh2_mac_',
+    'val_key': 'ssh_key_',
+    'val_cipher': 'ssh_cipher_',
+    'val_dh': 'dh_',
+    'val_ecdh': 'ssh_ecdhkex_',
+    'val_rsakex': 'ssh_rsakex_',
+    'val_prng': 'prng_',
+    'val_pcs': 'pcs_',
+    'val_pockle': 'pockle_',
+}
+method_lists = {t: [] for t in method_prefixes}
+
 class Value(object):
     def __init__(self, typename, ident):
-        self.typename = typename
-        self.ident = ident
-    def consumed(self):
-        self.ident = None
+        self._typename = typename
+        self._ident = ident
+        for methodname, function in method_lists.get(self._typename, []):
+            setattr(self, methodname,
+                    (lambda f: lambda *args: f(self, *args))(function))
+    def _consumed(self):
+        self._ident = None
     def __repr__(self):
-        return "Value({!r}, {!r})".format(self.typename, self.ident)
+        return "Value({!r}, {!r})".format(self._typename, self._ident)
     def __del__(self):
-        if self.ident is not None:
+        if self._ident is not None and not childprocess.already_terminated():
             try:
-                childprocess.funcall("free", [self.ident])
+                childprocess.funcall("free", [self._ident])
             except ChildProcessFailure:
                 # If we see this exception now, we can't do anything
                 # about it, because exceptions don't propagate out of
@@ -116,14 +132,22 @@ class Value(object):
                 # crashed for some other reason.)
                 pass
     def __long__(self):
-        if self.typename != "val_mpint":
+        if self._typename != "val_mpint":
             raise TypeError("testcrypt values of types other than mpint"
                             " cannot be converted to integer")
-        hexval = childprocess.funcall("mp_dump", [self.ident])[0]
+        hexval = childprocess.funcall("mp_dump", [self._ident])[0]
         return 0 if len(hexval) == 0 else int(hexval, 16)
     def __int__(self):
         return int(self.__long__())
 
+def marshal_string(val):
+    val = coerce_to_bytes(val)
+    assert isinstance(val, bytes), "Bad type for val_string input"
+    return "".join(
+        chr(b) if (0x20 <= b < 0x7F and b != 0x25)
+        else "%{:02x}".format(b)
+        for b in val)
+
 def make_argword(arg, argtype, fnname, argindex, to_preserve):
     typename, consumed = argtype
     if typename.startswith("opt_"):
@@ -131,56 +155,88 @@ def make_argword(arg, argtype, fnname, argindex, to_preserve):
             return "NULL"
         typename = typename[4:]
     if typename == "val_string":
-        arg = unicode_to_bytes(arg)
-        if isinstance(arg, bytes):
-            retwords = childprocess.funcall(
-                "newstring", ["".join("%{:02x}".format(b)
-                                      for b in bytevals(arg))])
-            arg = make_retvals([typename], retwords, unpack_strings=False)[0]
-            to_preserve.append(arg)
+        retwords = childprocess.funcall("newstring", [marshal_string(arg)])
+        arg = make_retvals([typename], retwords, unpack_strings=False)[0]
+        to_preserve.append(arg)
     if typename == "val_mpint" and isinstance(arg, numbers.Integral):
         retwords = childprocess.funcall("mp_literal", ["0x{:x}".format(arg)])
         arg = make_retvals([typename], retwords)[0]
         to_preserve.append(arg)
     if isinstance(arg, Value):
-        if arg.typename != typename:
+        if arg._typename != typename:
             raise TypeError(
                 "{}() argument {:d} should be {} ({} given)".format(
-                fnname, argindex, typename, arg.typename))
-        ident = arg.ident
+                fnname, argindex, typename, arg._typename))
+        ident = arg._ident
         if consumed:
-            arg.consumed()
+            arg._consumed()
         return ident
     if typename == "uint" and isinstance(arg, numbers.Integral):
         return "0x{:x}".format(arg)
+    if typename == "boolean":
+        return "true" if arg else "false"
     if typename in {
             "hashalg", "macalg", "keyalg", "cipheralg",
-            "dh_group", "ecdh_alg", "rsaorder"}:
-        arg = unicode_to_bytes(arg)
+            "dh_group", "ecdh_alg", "rsaorder", "primegenpolicy",
+            "argon2flavour", "fptype"}:
+        arg = coerce_to_bytes(arg)
         if isinstance(arg, bytes) and b" " not in arg:
             return arg
+    if typename == "mpint_list":
+        sublist = [make_argword(len(arg), ("uint", False),
+                                fnname, argindex, to_preserve)]
+        for val in arg:
+            sublist.append(make_argword(val, ("val_mpint", False),
+                                        fnname, argindex, to_preserve))
+        return b" ".join(coerce_to_bytes(sub) for sub in sublist)
     raise TypeError(
         "Can't convert {}() argument {:d} to {} (value was {!r})".format(
             fnname, argindex, typename, arg))
 
+def unpack_string(identifier):
+    retwords = childprocess.funcall("getstring", [identifier])
+    childprocess.funcall("free", [identifier])
+    return re.sub(b"%[0-9A-F][0-9A-F]",
+                  lambda m: bytes([int(m.group(0)[1:], 16)]),
+                  retwords[0])
+
+def unpack_mp(identifier):
+    retwords = childprocess.funcall("mp_dump", [identifier])
+    childprocess.funcall("free", [identifier])
+    return int(retwords[0], 16)
+
 def make_retval(rettype, word, unpack_strings):
     if rettype.startswith("opt_"):
         if word == b"NULL":
             return None
         rettype = rettype[4:]
     if rettype == "val_string" and unpack_strings:
-        retwords = childprocess.funcall("getstring", [word])
+        return unpack_string(word)
+    if rettype == "val_keycomponents":
+        kc = {}
+        retwords = childprocess.funcall("key_components_count", [word])
+        for i in range(int(retwords[0], 0)):
+            args = [word, "{:d}".format(i)]
+            retwords = childprocess.funcall("key_components_nth_name", args)
+            kc_key = unpack_string(retwords[0])
+            retwords = childprocess.funcall("key_components_nth_str", args)
+            if retwords[0] != b"NULL":
+                kc_value = unpack_string(retwords[0]).decode("ASCII")
+            else:
+                retwords = childprocess.funcall("key_components_nth_mp", args)
+                kc_value = unpack_mp(retwords[0])
+            kc[kc_key.decode("ASCII")] = kc_value
         childprocess.funcall("free", [word])
-        return re.sub(b"%[0-9A-F][0-9A-F]",
-                      lambda m: valbytes([int(m.group(0)[1:], 16)]),
-                      retwords[0])
+        return kc
     if rettype.startswith("val_"):
         return Value(rettype, word)
-    elif rettype == "uint":
+    elif rettype == "int" or rettype == "uint":
         return int(word, 0)
     elif rettype == "boolean":
         assert word == b"true" or word == b"false"
         return word == b"true"
+    elif rettype == "pocklestatus":
+        return word.decode("ASCII")
     raise TypeError("Can't deal with return value {!r} of type {!r}"
                     .format(word, rettype))
 
@@ -254,7 +310,14 @@ def _setup(scope):
                             consumed = True
                         arg = trim_argtype(arg)
                         argtypes.append((arg, consumed))
-                scope[function] = Function(function, rettypes, argtypes)
+                func = Function(function, rettypes, argtypes)
+                scope[function] = func
+                if len(argtypes) > 0:
+                    t = argtypes[0][0]
+                    if (t in method_prefixes and
+                        function.startswith(method_prefixes[t])):
+                        methodname = function[len(method_prefixes[t]):]
+                        method_lists[t].append((methodname, func))
 
 _setup(globals())
 del _setup
