@@ -13,6 +13,9 @@
 #define IDD_ABOUTBOX     111
 #define IDD_RECONF       112
 #define IDD_LICENCEBOX   113
+#define IDD_HK_ABSENT    114
+#define IDD_HK_WRONG     115
+#define IDD_HK_MOREINFO  116
 
 #define IDN_LIST        1001
 #define IDN_COPY        1002
@@ -29,6 +32,17 @@
 #define IDC_HELPBTN     1005
 #define IDC_ABOUT       1006
 
+#define IDC_HK_ICON       98
+#define IDC_HK_TITLE      99
+#define IDC_HK_ACCEPT   1001
+#define IDC_HK_ONCE     1000
+#define IDC_HK_FINGERPRINT 1002
+#define IDC_HK_MOREINFO 1003
+
+#define IDC_HKI_SHA256  1000
+#define IDC_HKI_MD5     1001
+#define IDC_HKI_PUBKEY  1002
+
 #define ID_CUSTOM_CHMFILE 2000
 #define TYPE_CUSTOM_CHMFILE 2000
 
