@@ -432,7 +432,7 @@ static IShellLink *make_shell_link(const char *appname,
          * behaviour change in which an argument string starting with
          * '@' causes the SetArguments method to silently do the wrong
          * thing. */
-        param_string = dupcat(" @", sessionname, NULL);
+        param_string = dupcat(" @", sessionname);
     } else {
         param_string = dupstr("");
     }
@@ -440,8 +440,7 @@ static IShellLink *make_shell_link(const char *appname,
     sfree(param_string);
 
     if (sessionname) {
-        desc_string = dupcat("Connect to PuTTY session '",
-                             sessionname, "'", NULL);
+        desc_string = dupcat("Connect to PuTTY session '", sessionname, "'");
     } else {
         assert(appname);
         desc_string = dupprintf("Run %.*s",
